module RottenParty

  class Client

    # https://github.com/jnunemaker/httparty
    include HTTParty

    # https://github.com/andreas/httparty-icebox
    include HTTParty::Icebox
    cache :store => 'file', :timeout => 600, :location => Dir.tmpdir

    # Base
    base_uri 'api.rottentomatoes.com/api/public'

    # Initialize the client
    # Options:
    #   - :api_version (default: 1.0)
    #   - :locale (default: en_UK)
    def initialize(api_key, api_version='1.0', options={:query => { :country => 'uk' }})
      self.class.base_uri "api.rottentomatoes.com/api/public/v#{api_version}"

      options.merge!({:query => {:apikey => api_key}})
      @global_options = options
    end

    # Find a specific film by it's identifier
    def find_film_by_id(id, options={})
      options = recurse_merge(options, @global_options)
      response = self.class.get("/movies/#{id}.json", options).parsed_response
      Hashie::Mash.new(response)
    end

    # Find a film by it's title
    def find_film_by_title(title, options={})
      options = recurse_merge(options, @global_options)
      options = recurse_merge(options, {:query => {:q => u(title)}})
      response = self.class.get("/movies.json", options).parsed_response
      Hashie::Mash.new(response["movies"].first)
    end

    # list
    # => 'box_office'
    # => 'in_theaters'
    # => 'opening'
    # => 'upcoming'
    #
    # directory
    # => 'dvds'
    # => 'movies'
    def find_movies_in_list(list, directory, options={})
      options = recurse_merge(options, @global_options)
      response = self.class.get("/lists/#{directory}/#{list}.json", options).parsed_response
      response["movies"].collect{ |movie| Hashie::Mash.new(movie) }
    end

    def recurse_merge(a,b)
      a.merge(b) do |_,x,y|
        (x.is_a?(Hash) && y.is_a?(Hash)) ? recurse_merge(x,y) : [*x,*y]
      end
    end
  end

end